﻿using FurnitureAssemblyBusinessLogic.OfficePackage;
using FurnitureAssemblyBusinessLogic.OfficePackage.HelperModels;
using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.BusinessLogicsContarcts;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyBusinessLogic
{
    public class ReportLogic : IReportLogic
    {        
        private readonly IFurnitureStorage _furnitureStorage;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;
        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToWord _saveToWord;
        private readonly AbstractSaveToPdf _saveToPdf;
        public ReportLogic(IFurnitureStorage furnitureStorage, IOrderStorage orderStorage, 
            AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf, IShopStorage shopStorage)
        {
            _furnitureStorage = furnitureStorage;
            _orderStorage = orderStorage;
            _saveToExcel = saveToExcel;
            _saveToWord = saveToWord;
            _saveToPdf = saveToPdf;
            _shopStorage = shopStorage;
        }
        /// <summary>
        /// Получение списка компонент с указанием, в каких изделиях используются
        /// </summary>
        /// <returns></returns>
        public List<ReportFurnitureComponentViewModel> GetFurnitureComponent()
        {
            var furnitures = _furnitureStorage.GetFullList();            
            var list = new List<ReportFurnitureComponentViewModel>();
            foreach (var furniture in furnitures)
            {
                var record = new ReportFurnitureComponentViewModel
                {
                    FurnitureName = furniture.FurnitureName,
                    Components = new List<Tuple<string, int>>(),
                    TotalCount = 0
                };
                foreach (var componentCount in furniture.FurnitureComponents.Values)
                {                    
                    record.Components.Add(new Tuple<string, int>(componentCount.Item1.ComponentName, componentCount.Item2));
                    record.TotalCount += componentCount.Item2;                    
                }
                list.Add(record);
            }
            return list;
        }
        /// <summary>
        /// Получение списка заказов за определенный период
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportOrdersViewModel
            {
                Id = x.Id,
                DateCreate = x.DateCreate,
                FurnitureName = x.FurnitureName,
                Sum = x.Sum,
                OrderStatus = x.Status.ToString()
            })
           .ToList();
        }
        /// <summary>
        /// Сохранение компонент в файл-Word
        /// </summary>
        /// <param name="model"></param>
        public void SaveFurnituresToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfoFurnitures
            {
                FileName = model.FileName,
                Title = "Список изделий",
                Furnitures = _furnitureStorage.GetFullList()
            });
        }      
        /// <summary>
        /// Сохранение компонент с указаеним продуктов в файл-Excel
        /// </summary>
        /// <param name="model"></param>
        public void SaveFurnitureComponentToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfoFurnitures
            {
                FileName = model.FileName,
                Title = "Список изделий",
                FurnitureComponents = GetFurnitureComponent()
            });
        }
        /// <summary>
        /// Сохранение заказов в файл-Pdf
        /// </summary>
        /// <param name="model"></param>
        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfoOrders
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }
        /// <summary>
        /// Сохранение магазинов в файл-Word
        /// </summary>
        /// <param name="model"></param>
        public void SaveShopsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateTableDoc(new WordInfoShops
            {
                FileName = model.FileName,
                Title = "Список магазинов",
                Shops = _shopStorage.GetFullList()
            });
        }

        public void SaveShopFurnituresToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfoShops
            {
                FileName = model.FileName,
                Title = "Список магазинов с изделиями",
                ShopFurnitures = GetShopFurnitures()
            });
        }

        public List<ReportShopFurnituresViewModel> GetShopFurnitures()
        {
            var shops = _shopStorage.GetFullList();
            var list = new List<ReportShopFurnituresViewModel>();
            foreach (var shop in shops)
            {
                var record = new ReportShopFurnituresViewModel
                {
                    ShopName = shop.ShopName,
                    Furnitures = new List<Tuple<string, int>>(),
                    TotalCount = 0
                };
                foreach (var furnitureCount in shop.Furnitures.Values)
                {
                    record.Furnitures.Add(new Tuple<string, int>(furnitureCount.Item1.FurnitureName, furnitureCount.Item2));
                    record.TotalCount += furnitureCount.Item2;
                }
                list.Add(record);
            }
            return list;
        }

        public List<ReportCountOrdersViewModel> GetCountOrders()
        {
            return _orderStorage.GetFullList().GroupBy(x => x.DateCreate.Date).Select(x => new ReportCountOrdersViewModel
            {
                DateCreate = x.Key,
                CountOrders = x.Count(),
                TotalSumOrders = x.Sum(y => y.Sum)
            }).ToList();
        }
        public void SaveCountOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfoCountOrders
            {
                FileName = model.FileName,
                Title = "Список объединенных по дате заказов",                
                CountOrders = GetCountOrders()
            });
        }
    }
}
