﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.BusinessLogicsContarcts;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyBusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        private readonly IFurnitureStorage _furnitureStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage, IFurnitureStorage furnitureStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
            _furnitureStorage = furnitureStorage;
        }
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ShopName:{ShopName}. Id:{ Id}", model.ShopName, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName:{ShopName}. Id:{ Id}", model?.ShopName, model?.Id);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
            }
            if (string.IsNullOrEmpty(model.Address))
            {
                throw new ArgumentNullException("Нет адреса магазина", nameof(model.Address));
            }
            if (model.DateOpening.Equals(null))
            {
                throw new ArgumentNullException("Нет даты открытия магазина", nameof(model.DateOpening));
            }
            if (model.MaxCount <= 0)
            {
                throw new ArgumentNullException("Нет вместимости магазина", nameof(model.DateOpening));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}. Address:{ Address}. DateOpening:{ DateOpening}. Id: { Id}", model.ShopName, model.Address, model.DateOpening, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel { ShopName = model.ShopName });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

        public bool AddFurniture(ShopBindingModel model, FurnitureBindingModel furnitureModel, int count)
        {
            var shop = _shopStorage.GetElement(new ShopSearchModel { Id = model.Id });
            var furniture = _furnitureStorage.GetElement(new FurnitureSearchModel { Id = furnitureModel.Id });

            if (shop == null || furniture == null)
            {
                return false;
            }

            if (GetCountFreePlaces(model.Id) < count)
            {
                return false;
            }

            if (shop.Furnitures.ContainsKey(furniture.Id))
            {
                int prev_count = shop.Furnitures[furniture.Id].Item2;
                shop.Furnitures[furniture.Id] = (furniture, prev_count + count);
            }
            else
            {
                shop.Furnitures[furniture.Id] = (furniture, count);
            }
            model.Furnitures = shop.Furnitures;
            model.DateOpening = shop.DateOpening;
            model.MaxCount = shop.MaxCount;
            model.Address = shop.Address;
            model.ShopName = shop.ShopName;          
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Replenishment operation failed");
                return false;
            }
            return true;
        }

        private int GetCountFreePlaces(int ShopId)
        {
            var shop = ReadElement(new ShopSearchModel { Id = ShopId });
            if (shop == null)
                return 0;
            int count = shop.MaxCount;
            foreach (var f in shop.Furnitures)
            {
                count -= f.Value.Item2;
                if (count <= 0)
                {
                    return 0;
                }
            }
            return count;
        }

        private int GetCountFreePlaces_All()
        {
            return _shopStorage.GetFullList().Select(x => GetCountFreePlaces(x.Id)).Sum();
        }

        public bool AddFurnituresAtShops(FurnitureBindingModel furnitureModel, int count)
        {
            if (GetCountFreePlaces_All() < count)
            {
                return false;
            }

            foreach (var shop in _shopStorage.GetFullList())
            {
                int countShop = GetCountFreePlaces(shop.Id);
                if (countShop >= count)
                {
                    AddFurniture(new() { Id = shop.Id }, furnitureModel, count);
                    break;
                }
                else
                {
                    AddFurniture(new() { Id = shop.Id }, furnitureModel, countShop);
                }
                count -= countShop;
            }
            return true;
        }

        public bool Sell(FurnitureBindingModel furnitureBindingModel, int count)
        {
            return _shopStorage.Sell(furnitureBindingModel, count);
        }
    }
}
